# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation builder.
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))
def setup(app):
    app.add_css_file('css/custom.css')


# -- Project information -----------------------------------------------------

project = u'The Logtalk Handbook'
copyright = u'1998-2024, Paulo Moura'
author = u'Paulo Moura'

# The short X.Y version
version = '3.81.0'
# The full version, including alpha/beta/rc tags
release = u'v3.81.0'


# -- General configuration ---------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.intersphinx'
]

intersphinx_mapping = {'apis': ('../docs/','../../docs/objects.inv')}

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path .
exclude_patterns = [u'_build', '_static', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
# pygments_style = 'sphinx'
pygments_style = 'friendly'

highlight_language = 'logtalk'


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'
# html_theme = 'groundwork'
# html_theme = 'rtcat_sphinx_theme'
# import rtcat_sphinx_theme
# html_theme_path = [rtcat_sphinx_theme.get_html_theme_path()]

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}
html_logo = 'logtalk.gif'
# html_favicon = 'logtalk.ico'
# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']
html_extra_path = ['android-chrome-192x192.png', 'android-chrome-512x512.png', 'apple-touch-icon.png', 'browserconfig.xml', 'favicon-16x16.png', 'favicon-32x32.png', 'favicon.ico', 'mstile-150x150.png', 'safari-pinned-tab.svg', 'site.webmanifest']
# html_last_updated_fmt = ''

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# The default sidebars (for documents that don't match any pattern) are
# defined by theme itself.  Builtin themes are using these templates by
# default: ``['localtoc.html', 'relations.html', 'sourcelink.html',
# 'searchbox.html']``.
#
# html_sidebars = {}
html_context = {
    "display_github": True, # Integrate GitHub
    "github_user": "LogtalkDotOrg", # Username
    "github_repo": "logtalk3", # Repo name
    "github_version": "master", # Version
    "conf_py_path": "/manuals/sources/", # Path in the checkout to the docs root
}

# -- Options for HTMLHelp output ---------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'TheLogtalkHandbook-' + version


# -- Options for LaTeX output ------------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    'preamble': r'''
        \usepackage{charter}
        \usepackage[defaultsans]{lato}
        \usepackage{inconsolata}
        \usepackage[utf8]{inputenc}
        \DeclareUnicodeCharacter{03C9}{$\omega$}
    ''',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    ('index_latexpdf', 'TheLogtalkHandbook-' + version + '.tex', u'The Logtalk Handbook',
     u'Paulo Moura', 'manual'),
]


# -- Options for manual page output ------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'logtalk', u'The Logtalk Handbook',
     [author], 1)
]


# -- Options for Texinfo output ----------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'TheLogtalkHandbook-' + version, u'The Logtalk Handbook',
     u'Paulo Moura', u'The Logtalk Handbook', 'Declarative object-oriented logic programming language.',
     'Programming Languages'),
]


# -- Options for Epub output -------------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project + '\n\n' + release
epub_author = author
epub_publisher = author
epub_copyright = copyright

epub_basename = 'TheLogtalkHandbook-' + version
# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A list of files that should not be packed into the epub file.
epub_exclude_files = ['search.html']


# pdf_documents = [('index', u'handbook', u'The Logtalk Handbook', u'Paulo Moura'),]
